//---------------------------------------------------------------------------
#include <vcl.h>
#include <math.h>
#pragma hdrstop
#include "rtklib.h"
#include "graph.h"

#define MINSIZE		10
#define MINSCALE	1E-4
#define MAXSCALE	1E4
//---------------------------------------------------------------------------
TGraph::TGraph(TPaintBox *parent)
{
	Parent=parent; X=Y=0; Width=parent->Width; Height=parent->Width;
	XCent =YCent =0.0;			// center coordinate (unit)
	XScale=YScale=20.0; 		// scale factor (unit/pixel)
	Box=1;						// show box (0:off,1:on)
	Fit=1;						// fit scale on resize (0:off,1:on):
	XGrid=YGrid=1;				// show grid (0:off,1:on)
	XTick=YTick=0.0;			// grid interval (unit) (0:auto)
	XLPos=YLPos=1;				// grid label pos (0:off,1:outer,2:inner,
								// 3:outer-rot,4:inner-rot,5/6:time)
	Title=XLabel=YLabel="";		// lable string ("":no label)
	Color[0]=Parent->Color;		// background color
	Color[1]=clGray;			// grid color
	Color[2]=clBlack;			// title/label color
}
//---------------------------------------------------------------------------
int TGraph::ToPoint(double x, double y, TPoint &p)
{
	x=X+(Width-1)/2.0+(x-XCent)/XScale;
	y=Y+(Height-1)/2.0-(y-YCent)/YScale;
	p.x=(int)floor(x+0.5); p.y=(int)floor(y+0.5);
	return X<=x&&x<=X+Width-1&&Y<=y&&y<=Y+Height-1;
}
//---------------------------------------------------------------------------
void TGraph::ToPos(TPoint p, double &x, double &y)
{
	x=XCent+(p.x-X-(Width-1)/2.0)*XScale;
	y=YCent-(p.y-Y-(Height-1)/2.0)*YScale;
}
//---------------------------------------------------------------------------
void TGraph::SetPos(TPoint p1, TPoint p2)
{
	int w=p2.x-p1.x+1,h=p2.y-p1.y+1;
	if (w<MINSIZE) w=MINSIZE; if (h<MINSIZE) h=MINSIZE;
	if (Fit) {
		XScale*=(double)(Width-1)/(w-1);
		YScale*=(double)(Height-1)/(h-1);
	}
	X=p1.x; Y=p1.y; Width=w; Height=h;
};
//---------------------------------------------------------------------------
void TGraph::GetPos(TPoint &p1, TPoint &p2)
{
	p1.x=X; p1.y=Y; p2.x=X+Width-1; p2.y=Y+Height-1;
};
//---------------------------------------------------------------------------
void TGraph::SetCent(double x, double y)
{
	XCent=x; YCent=y;
}
//---------------------------------------------------------------------------
void TGraph::GetCent(double &x, double &y)
{
	x=XCent; y=YCent;
}
//---------------------------------------------------------------------------
void TGraph::SetScale(double xs, double ys)
{
	if (xs<MINSCALE) xs=MINSCALE; else if (MAXSCALE<xs) xs=MAXSCALE;
	if (ys<MINSCALE) ys=MINSCALE; else if (MAXSCALE<ys) ys=MAXSCALE;
	XScale=xs; YScale=ys;
}
//---------------------------------------------------------------------------
void TGraph::GetScale(double &xs, double &ys)
{
	xs=XScale; ys=YScale;
}
//---------------------------------------------------------------------------
void TGraph::SetLim(const double *xl, const double *yl)
{
	if (xl[0]<xl[1]) {XCent=(xl[0]+xl[1])/2.0; XScale=(xl[1]-xl[0])/Width;}
	if (yl[0]<yl[1]) {YCent=(yl[0]+yl[1])/2.0; YScale=(yl[1]-yl[0])/Height;}
}
//---------------------------------------------------------------------------
void TGraph::GetLim(double *xl, double *yl)
{
	TPoint p0(X,Y),p1(X+Width-1,Y+Height-1);
	ToPos(p0,xl[0],yl[1]); ToPos(p1,xl[1],yl[0]);
}
//---------------------------------------------------------------------------
void TGraph::SetTick(double xt, double yt)
{
	XTick=xt; YTick=yt;
}
//---------------------------------------------------------------------------
void TGraph::GetTick(double &xt, double &yt)
{
	xt=XTick>0.0?XTick:(XLPos>=5?AutoTickTime(XScale):AutoTick(XScale));
	yt=YTick>0.0?YTick:AutoTick(YScale);
}
//---------------------------------------------------------------------------
double TGraph::AutoTick(double scale)
{
	double t[]={1.0,2.0,5.0,10.0},tick=30.0*scale;
	double order=pow(10.0,floor(log10(tick)));
	for (int i=0;i<4;i++) if (tick<=t[i]*order) return t[i]*order;
	return 10.0;
}
//---------------------------------------------------------------------------
double TGraph::AutoTickTime(double scale)
{
	double t[]={1.0,5.0,10.0,30.0,90.0,300.0,900.0,1800.0,3600.0,7200.0,
	            10800.0,21600.0,43200.0};
	double tick=60.0*scale;
	for (int i=0;i<sizeof(t)/sizeof(*t);i++) if (tick<=t[i]) return t[i];
	return 86400.0;
}
//---------------------------------------------------------------------------
AnsiString TGraph::NumText(double x, double dx)
{
	AnsiString s;
	int n=(int)(0.9-log10(dx));
	return s.sprintf("%.*f",n<0?0:n,x);
}
//---------------------------------------------------------------------------
AnsiString TGraph::TimeText(double x, double dx)
{
    AnsiString s;
    char str[64];
    time2str(gpst2time(0,x),str,0);
    int w=dx<60.0?8:5;
    return s.sprintf("%*.*s",w,w,str+11);
}
//---------------------------------------------------------------------------
void TGraph::DrawGrid(double xt, double yt)
{
	TCanvas *c=Parent->Canvas;
	double xl[2],yl[2];
	TPoint p;
	GetLim(xl,yl);
	c->Pen->Color=Color[1]; c->Brush->Style=bsClear;
	if (XGrid) {
		for (int i=(int)ceil(xl[0]/xt);i*xt<=xl[1];i++) {
			ToPoint(i*xt,0.0,p);
			c->Pen->Style=i!=0?psDot:psSolid;
			c->MoveTo(p.x,Y); c->LineTo(p.x,Y+Height-1);
		}
	}
	if (YGrid) {
		for (int i=(int)ceil(yl[0]/yt);i*yt<=yl[1];i++) {
			ToPoint(0.0,i*yt,p);
			c->Pen->Style=i!=0?psDot:psSolid;
			c->MoveTo(X,p.y); c->LineTo(X+Width-1,p.y);
		}
	}
}
//---------------------------------------------------------------------------
void TGraph::DrawGridLabel(double xt, double yt)
{
	double xl[2],yl[2];
	TPoint p;
	GetLim(xl,yl);
	if (XLPos) {
		for (int i=(int)ceil(xl[0]/xt);i*xt<=xl[1];i++) {
			ToPoint(i*xt,yl[0],p); if (XLPos==1) p.y-=1;
			if (XLPos<=4) {
				int ha=XLPos<=2?0:(XLPos==3?2:1),va=XLPos>=3?0:(XLPos==1?2:1);
				DrawText(p,NumText(i*xt,xt),Color[2],ha,va,XLPos>=3?90:0);
			}
			else if (XLPos==6) {
				DrawText(p,TimeText(i*xt,xt),Color[2],0,2,0);
			}
		}
	}
	if (YLPos) {
		for (int i=(int)ceil(yl[0]/yt);i*yt<=yl[1];i++) {
			ToPoint(xl[0],i*yt,p);
			int ha=YLPos>=3?0:(YLPos==1?2:1),va=YLPos<=2?0:(YLPos==3?1:2);
			DrawText(p,NumText(i*yt,yt),Color[2],ha,va,YLPos>=3?90:0);
		}
	}
}
//---------------------------------------------------------------------------
void TGraph::DrawBox(void)
{
	TCanvas *c=Parent->Canvas;
	if (Box) {
		c->Pen->Color=Color[1]; c->Pen->Style=psSolid; c->Brush->Style=bsClear;
		c->Rectangle(X,Y,X+Width-1,Y+Height-1);
	}
}
//---------------------------------------------------------------------------
void TGraph::DrawLabel(void)
{
	if (XLabel!="") {
		TPoint p(X+Width/2,Y+Height+(XLPos%2?10:2));
		DrawText(p,XLabel,Color[2],0,2,0);
	}
	if (YLabel!="") {
		TPoint p(X-(YLPos%2?20:2),Y+Height/2);
		DrawText(p,YLabel,Color[2],0,1,90);
	}
	if (Title!="") {
		TPoint p(X+Width/2,Y-1);
		DrawText(p,Title,Color[2],0,1,0);
	}
}
//---------------------------------------------------------------------------
void TGraph::DrawAxis(void)
{
	TCanvas *c=Parent->Canvas;
	double xt,yt;
	GetTick(xt,yt);
	c->Pen->Color=Color[0]; c->Pen->Style=psSolid;
	c->Brush->Color=Color[0]; c->Brush->Style=bsSolid;
	c->Rectangle(X,Y,X+Width-1,Y+Height-1);
	DrawGrid(xt,yt);
	if (xt/XScale<50.0&&XLPos<=2) xt*=XLPos==5?4.0:2.0;
	if (yt/YScale<50.0&&YLPos>=3) yt*=2.0;
	DrawGridLabel(xt,yt);
	DrawBox();
	DrawLabel();
}
//---------------------------------------------------------------------------
void TGraph::RotPoint(TPoint *ps, int n, TPoint pc, int rot, TPoint *pr)
{
	for (int i=0;i<n;i++) {
		pr[i].x=pc.x+(int)floor(ps[i].x*cos(rot*D2R)-ps[i].y*sin(rot*D2R)+0.5);
		pr[i].y=pc.y-(int)floor(ps[i].x*sin(rot*D2R)+ps[i].y*cos(rot*D2R)+0.5);
	}
}
//---------------------------------------------------------------------------
void TGraph::DrawMark(TPoint p, int mark, TColor color, int size, int rot)
{
	TCanvas *c=Parent->Canvas;
	if (size<1) size=1;
	int n,s=size/2;
	int x1=p.x-s,x2=x1+size+1,y1=p.y-s,y2=y1+size+1;
	int xs1[]={-7,0,-7,0},ys1[]={2,0,-2,0};
	int xs2[]={-1,-1,-1,1,1,1},ys2[]={-1,1,0,0,-1,1};
	int xs3[]={3,-4,0,0,0,-8,8},ys3[]={0,5,20,-20,-10,-10,-10};
	TPoint ps[32],pr[32],pd(0,size/2+12),pt;
	c->Pen->Color=color; c->Pen->Style=psSolid; c->Brush->Color=color;
	switch (mark) {
	case 0: c->Brush->Style=bsSolid; c->Ellipse(x1,y1,x2,y2); return; // dot
	case 1: c->Brush->Style=bsClear; c->Ellipse(x1,y1,x2,y2); return; // circle
	case 2: c->Brush->Style=bsClear; c->Rectangle(x1,y1,x2,y2); return; // rect
	case 3: c->Brush->Style=bsClear;
			c->MoveTo(x1,y1); c->LineTo(x2,y2); // cross
			c->MoveTo(x1,y2); c->LineTo(x2,y1); return;
	case 10:n=6; // arrow
			ps[0].x=-size/2; ps[0].y=0; ps[1].x=size/2; ps[1].y=0;
			for (int i=2;i<n;i++) {
				ps[i].x=size/2+xs1[i-2]; ps[i].y=ys1[i-2];
			}
			break;
	case 11: case 12: // hscale,vscale
			n=6;
			for (int i=0;i<n;i++) {
				int x=xs2[i]*size/2,y=ys2[i]*5;
				ps[i].x=mark==11?x:y; ps[i].y=mark==11?y:x;
			}
			break;
	case 13:n=7; // dir-arrow
			for (int i=0;i<n;i++) {
				ps[i].x=xs3[i]*size/40; ps[i].y=ys3[i]*size/40;
			}
			RotPoint(&pd,1,p,rot,&pt);
			DrawText(pt,"N",color,0,0,rot);
			break;
	default:return;
	}
	c->Brush->Style=bsClear;
	RotPoint(ps,n,p,rot,pr);
	DrawPoly(pr,n,color,0);
}
//---------------------------------------------------------------------------
void TGraph::DrawMark(double x, double y, int mark, TColor color, int size,
	int rot)
{
	TPoint p;
	if (ToPoint(x,y,p)) DrawMark(p,mark,color,size,rot);
}
//---------------------------------------------------------------------------
void TGraph::DrawText(TPoint p, AnsiString str, TColor color, int ha, int va,
	int rot)
{
	// ha=0:cent,1:left,2:right,va=0:cent,1:bottom,2:top,rot=rotation(deg)
	TCanvas *c=Parent->Canvas;
	LOGFONT lf={0};
	lf.lfHeight=c->Font->Height;
	lf.lfCharSet=c->Font->Charset;
	strcpy(lf.lfFaceName,c->Font->Name.c_str());
	lf.lfEscapement=lf.lfOrientation=rot*10;
	c->Font->Handle=CreateFontIndirect(&lf);
	TSize off=c->TextExtent(str);
	TPoint ps,pr;
	ps.x=ha==0?-off.cx/2:(ha==1?3:-off.cx-3);
	ps.y=va==0?off.cy/2+1:(va==1?off.cy+1:-2);
	RotPoint(&ps,1,p,rot,&pr);
	c->Font->Color=color;
	c->Brush->Style=bsClear;
	c->TextOut(pr.x,pr.y,str);
}
//---------------------------------------------------------------------------
void TGraph::DrawText(double x, double y, AnsiString str, TColor color,
	int ha, int va, int rot)
{
	TPoint p;
	ToPoint(x,y,p);
	DrawText(p,str,color,ha,va,rot);
}
//---------------------------------------------------------------------------
int TGraph::OnAxis(TPoint p)
{
	// area code : [5 4 6; 1 0 2; 9 8 10];
	int xmin=X,xmax=X+Width-1,ymin=Y,ymax=Y+Height-1;
	return (p.x<xmin?1:(p.x<=xmax?0:2))+(p.y<ymin?4:(p.y<=ymax?0:8));
}
//---------------------------------------------------------------------------
int TGraph::ClipPoint(TPoint *p0, int area, TPoint *p1)
{
	int x,y,xmin=X,xmax=X+Width-1,ymin=Y,ymax=Y+Height-1;
	if (area&1) { // left
		y=p0->y+(p1->y-p0->y)*(xmin-p0->x)/(p1->x-p0->x);
		if (ymin<=y&&y<=ymax) {p0->x=xmin; p0->y=y; return 1;}
	}
	if (area&2) { // right
		y=p0->y+(p1->y-p0->y)*(xmax-p0->x)/(p1->x-p0->x);
		if (ymin<=y&&y<=ymax) {p0->x=xmax; p0->y=y; return 1;}
	}
	if (area&4) { // upper
		x=p0->x+(p1->x-p0->x)*(ymin-p0->y)/(p1->y-p0->y);
		if (xmin<=x&&x<=xmax) {p0->x=x; p0->y=ymin; return 1;}
	}
	if (area&8) { // lower
		x=p0->x+(p1->x-p0->x)*(ymax-p0->y)/(p1->y-p0->y);
		if (xmin<=x&&x<=xmax) {p0->x=x; p0->y=ymax; return 1;}
	}
	return 0;
}
//---------------------------------------------------------------------------
void TGraph::DrawPoly(TPoint *p, int n, TColor color, int style)
{
	TCanvas *c=Parent->Canvas;
	TPenStyle ps[]={psSolid,psDash,psDot,psDashDot,psDashDotDot};
	c->Pen->Color=color; c->Pen->Style=ps[style]; c->Brush->Style=bsClear;
	int i,j,area0=11,area1;
	for (i=j=0;j<n;j++,area0=area1) {
		if ((area1=OnAxis(p[j]))==area0) continue;
		if (!area1) i=j; else if (!area0) c->Polyline(p+i,j-i-1);
		if (j<=0||(area0&area1)) continue;
		TPoint pc[2]={p[j-1],p[j]};
		if (area0&&!ClipPoint(pc,  area0,p+j  )) continue;
		if (area1&&!ClipPoint(pc+1,area1,p+j-1)) continue;
		c->Polyline(pc,1);
	}
	if (!area0) c->Polyline(p+i,j-i-1);
}
//---------------------------------------------------------------------------
void TGraph::DrawPoly(double *x, double *y, int n, TColor color, int style)
{
	TPoint *p=new TPoint[n];
	for (int i=0;i<n;i++) ToPoint(x[i],y[i],p[i]);
	DrawPoly(p,n,color,style);
	delete [] p;
}
//---------------------------------------------------------------------------
