function ploterr2(file,q)
% plot rover position error
% args  : file   rover position file
%         qval   Q value (0:all,1-6: only plot qval) [0]

if strncmp(file,'cb_',3), feval(file); return; end
if nargin<2, q=0; end, if ischar(q), q=str2num(q); end

p=textread(file,'%s',7,'delimiter','\n');
rref=sscanf(p{end},'%% ref pos  : %f %f %f');

p=textread(file,'','commentstyle','matlab');
if isempty(p) return; elseif p(1,1)>p(end,1), p=flipud(p); end

pos={ % GSI F2 (2004/1/1)
'0759',-3.9762192010E+06,3.3823716036E+06,3.6525111482E+06
'0230',-3.9430393004E+06,3.4104618858E+06,3.6622739362E+06
'0625',-3.9139496962E+06,3.5045131515E+06,3.6046189324E+06
'0627',-3.9572346126E+06,3.3103689032E+06,3.7375298472E+06
'2106',-3.9520664880E+06,3.4255647198E+06,3.6385244749E+06
'3042',-3.9469331600E+06,3.4176825340E+06,3.6513922332E+06
'3064',-3.9161393260E+06,3.4071465109E+06,3.6945775061E+06
'5105',-3.9557698093E+06,3.4180202804E+06,3.6416025963E+06
};
rr=[];
for n=1:size(pos,1), if findstr(file,pos{n,1}), rr=[pos{n,2:4}]; end, end
if isempty(rr), rr=mean(p(~isnan(p(:,1)),2:4),1)'; end % reference by average positon

% plot errors
figure('color','w','renderer','painters'); hold on, grid on, box on
if q==0, i=1:size(p,1); else i=find(p(:,5)==q); end
if ~isempty(i), plot_pos(file,q,p(1,1),p(end,1),p(2,1)-p(1,1),p(i,1),p(i,2:4),rr',rref,'.-'); end
moveaxis;

% plot errors ------------------------------------------------------------------
function plot_pos(file,q,ts,te,ti,time,xs,rrov,rref,style)
rrov=epos(rrov);
rref=epos(rref);
for n=1:size(xs,1), xs(n,:)=epos(xs(n,:)')'; end
font='Times New Roman';
[pos,E]=gpos(rrov);
err=E*(xs(:,1:3)'-repmat(rrov,1,size(xs,1)));

%i=find(abs(err(3,:))>0.1);
%err(:,i)=nan;

if q>=4, yl=10; else yl=0.2; end
plot(time,err',style');
set(gca,'position',[0.1,0.1,0.85,0.82],'fontname',font);
xlim([ts,te]); ylim([-yl,yl]);
xlabel('Time (sec)'); ylabel('Position Error (m)');
if isempty(rref), s=''; else s=sprintf('BASELINE=%.2fkm,',norm(rrov-rref)/1E3); end
title(sprintf('POSITION ERROR : %s (%sQ=%d)',file,s,q),'fontname',font);
set(legend({'East (m)','North (m)','Up (m)'}),'fontname',font);
for n=1:3, rms(n)=sqrt(mean(err(n,~isnan(err(n,:))).^2)); end
r=size(xs,1)/((te-ts)/ti+1)*100;
s=sprintf('  RMSE : E:%7.4fm N:%7.4fm U:%7.4fm RATE:%5.1f%%',rms,r);
text(0,0.98,s,'units','normalized','horizontal','left','vertical','top',...
     'fontname',font);
disp([file,' :',s]);

% ecef to wgs84 / rotation matrix ----------------------------------------------
function [pos,E]=gpos(r)
RE=6378137; F=1/298.257223563;
e2=F*(2-F); r2=r(1:2)'*r(1:2); z=r(3); zk=0; n=RE;
while abs(z-zk)>1E-4
    sinp=z/sqrt(r2+z^2);
    n=RE/sqrt(1-e2*sinp^2);
    zk=z; z=r(3)+n*e2*sinp;
end
pos=[atan(z/sqrt(r2)),atan2(r(2),r(1)),sqrt(r2+z^2)-n];
sinp=sin(pos(1)); cosp=cos(pos(1)); sinl=sin(pos(2)); cosl=cos(pos(2));
E=[-sinl,cosl,0;-sinp*cosl,-sinp*sinl,cosp;cosp*cosl,cosp*sinl,sinp];
pos(1:2)=pos(1:2)*180/pi;

% llh to xyz -------------------------------------------------------------------
function r=epos(pos)
if isempty(pos), r=[nan;nan;nan]; return, end
RE=6378137; F=1/298.257223563;
pos(1:2)=pos(1:2)*pi/180;
e2=F*(2.0-F); v=RE/sqrt(1.0-e2*sin(pos(1))^2);
r=[(v+pos(3))*cos(pos(1))*cos(pos(2));
   (v+pos(3))*cos(pos(1))*sin(pos(2));
   (v*(1.0-e2)+pos(3))*sin(pos(1))];

% move axis --------------------------------------------------------------------
function moveaxis
p.mov=0; p.pnt=[0,0]; p.xl=[0,1]; p.yl=[0,1];
set(gca,'userdata',p);
set(gcf,'windowbuttondownfcn',[mfilename,' cb_btndown'],...
    'windowbuttonmotionfcn',[mfilename,' cb_btndrag'],...
    'windowbuttonupfcn',[mfilename,' cb_btnup']);

function cb_btndown
ax=get(gcf,'currentaxes'); if isempty(ax), return; end
p=get(ax,'userdata'); if ~isstruct(p)|~isfield(p,'mov'), return; end
siz=get(gcf,'position');
pos=get(ax,'position');
pos=[siz(3:4).*pos(1:2),siz(3:4).*(pos(1:2)+pos(3:4))];
p.pnt=get(gcf,'currentpoint');
if     abs(p.pnt(2)-pos(2))<10&pos(1)<p.pnt(1)&p.pnt(1)<pos(3), p.mov=1;
elseif abs(p.pnt(1)-pos(1))<10&pos(2)<p.pnt(2)&p.pnt(2)<pos(4), p.mov=2;
else return; end
if strcmp(get(gcf,'selectiontype'),'alt'), p.mov=p.mov+2; end % right button
p.xl=xlim; p.yl=ylim;
set(ax,'userdata',p);
cur={'right','top'};
if p.mov<=2, set(gcf,'pointer',cur{p.mov}); else setcurzoom(p.mov==3); end

function cb_btndrag
ax=get(gcf,'currentaxes'); if isempty(ax), return; end
p=get(ax,'userdata'); if ~isstruct(p)|~isfield(p,'mov')|p.mov<=0, return; end
siz=get(gcf,'position');
pos=get(ax,'position');
dp=(get(gcf,'currentpoint')-p.pnt)./siz(3:4)./pos(3:4);
switch p.mov
case 1, p.xl=p.xl-(p.xl(2)-p.xl(1))*dp(1); % move x axis
case 2, p.yl=p.yl-(p.yl(2)-p.yl(1))*dp(2); % move y axis
case 3, p.xl=p.xl(1)+(p.xl(2)-p.xl(1))*(0.5+[-0.5,0.5]*0.1^dp(1)); % zoom x axis
case 4, p.yl=p.yl(1)+(p.yl(2)-p.yl(1))*(0.5+[-0.5,0.5]*0.1^dp(2)); % zoom y axis
end
xlim(p.xl); ylim(p.yl);

function cb_btnup
ax=get(gcf,'currentaxes'); if isempty(ax), return; end
p=get(ax,'userdata'); if ~isstruct(p)|~isfield(p,'mov')|p.mov<=0, return; end
p.mov=0;
set(ax,'userdata',p);
set(gcf,'pointer','arrow');

function setcurzoom(opt)
o=nan;
p=[o,o,o,o,o,o,o,2,1,2,o,o,o,o,o,o
   o,o,o,o,o,o,2,1,1,1,2,o,o,o,o,o
   o,o,o,o,o,2,1,1,1,1,1,2,o,o,o,o
   o,o,o,o,2,1,1,1,1,1,1,1,2,o,o,o
   o,o,o,2,1,1,1,1,1,1,1,1,1,2,o,o
   o,o,o,2,2,2,2,2,1,2,2,2,2,2,o,o
   o,o,o,o,o,o,o,2,1,2,o,o,o,o,o,o
   o,o,o,o,o,o,o,2,1,2,o,o,o,o,o,o
   o,o,o,o,o,2,2,2,1,2,2,2,o,o,o,o
   o,o,o,o,o,2,1,1,1,1,1,2,o,o,o,o
   o,o,o,o,o,2,2,2,1,2,2,2,o,o,o,o
   o,o,o,o,o,o,o,2,1,2,o,o,o,o,o,o
   o,o,o,o,o,2,2,2,1,2,2,2,o,o,o,o
   o,o,o,o,o,2,1,1,1,1,1,2,o,o,o,o
   o,o,o,o,o,o,2,1,1,1,2,o,o,o,o,o
   o,o,o,o,o,o,o,2,1,2,o,o,o,o,o,o];
if opt, p=fliplr(p'); end
set(gcf,'pointer','custom','pointershapecdata',p,'pointershapehotspot',[9,9])
