function plottrk(file,qval,opt,file2,qval2)
% plot rover track
% args  : file   rover position file
%         qval   Q value (0:all,1-6: only plot qval) [0]
%         op     options (0:no,1:with arrow, 2:with time label) [1]
%         file2  compared position file [none]
%         qval2  Q value for file2 [0]

if strncmp(file,'cb_',3), feval(file); return; end
if nargin<2, qval=0;   end, if ischar(qval), qval=str2num(qval); end
if nargin<3, opt=1;    end, if ischar(opt), opt=str2num(opt); end
if nargin<4, file2=''; end
if nargin<5, qval2=0;  end, if ischar(qval2), qval2=str2num(qval2); end

bcolor='w'; fcolor='k'; tcolor='b'; font='Times New Roman';

figure('color',bcolor); hold on, grid on, box on

p=[]; posb=[];
if exist(file)
    [p,posb]=plotfile(file,qval,1,tcolor,font,opt,posb);
end
if ~isempty(file2)&exist(file2)
    plotfile(file2,qval2,0,tcolor,font,0,posb);
end
ti='';
if ~isempty(posb)
    plot(0,0,'o','color',tcolor,'markersize',5);
    text(0,0,'  Reference Pos','fontname',font,'clipping','on');
    b=sqrt(sum(p.^2,2))/1E3;
    ti=sprintf(' (BASELINE=%.1f - %.1fkm)',min(b),max(b));
end
title(['ROVER TRACK (m) : ',file,' ',file2,ti],'fontname',font);
set(gca,'color','none','xcolor',fcolor,'ycolor',fcolor,'fontname',font);
moveaxis;

mark=[0.07,0.07,0.06,0.08,nan,0.05,0.09;0.78,0.93,0.88,0.86,nan,0.82,0.82];
ax=gca; axes('position',get(gca,'position'));
plot(mark(1,:),mark(2,:),'-','color',fcolor);
text(0.07,0.93,'N','horizontal','center','vertical','bottom','fontname',font);
xlim([0,1]), ylim([0,1]); axis off; axes(ax);

set(gcf,'paperpositionmode','auto','resizefcn',[mfilename,' cb_resize'],...
    'windowbuttondownfcn',[mfilename,' cb_btndown'],...
    'windowbuttonmotionfcn',[mfilename,' cb_btndrag'],...
    'windowbuttonupfcn',[mfilename,' cb_btnup']);
legend('ResizeLegend');

% plot track file --------------------------------------------------------------
function [p,posb]=plotfile(file,qval,flag,color,font,opt,posb)
msiz=5; mw=2;
if flag, style='-'; mark='o'; else style=':'; color=[0.7,0.7,0.7]; mark='x'; end
labels={'FIX','FLOAT','FLOAT','DGPS','SINGLE','NONE'};
cs={'r',[1,0.7,0],[0,1,0],[0,0.7,0],[0,0.7,0],'b'};

s=textread(file,'%s','delimiter','\n');

t=[]; p=[]; q=[]; rb=[];
for n=1:length(s)
    if isempty(s{n}), continue; end
    if s{n}(1)=='%'|s{n}(1)==';'|s{n}(1)=='#'
        pos=sscanf(s{n},'%*s ref pos  : %f %f %f')';
        if ~isempty(pos), posb=pos; end
        continue;
    end
    v=sscanf(s{n},'%f/%f/%f %f:%f:%f %f %f %f %f %f %f %f %f');
    
    if length(v)>6, tt=v(4:6)'*[3600;60;1]; v=v(7:end);
    else tt=mod(v(1),86400); v=sscanf(s{n},'%*f %f %f %f %f %f %f %f %f'); end
    
    if length(v)>=5&all(mod(v([1:2,4:5]),1)==0) % dd mm ss form
        v(1)=v(1:3)'*[1;1/60;1/3600]; v(2)=v(4:6)'*[1;1/60;1/3600]; v(3:4)=v(7:8);
    end
    if isempty(rb)
        if isempty(posb), [rb,E]=epos(v(1:3)); else [rb,E]=epos(posb); end
    end
    t=[t;tt]; p=[p;(E*(epos(v(1:3))-rb))']; q=[q;v(4)];
end
h=[]; label={};
if qval==0
    plotpath(t,p,style,color,font,opt);
    for n=1:length(labels)
        i=find(q==n); if isempty(i), continue; end
        h=[h,plot(p(i,1),p(i,2),mark,'color',cs{n},'markersize',msiz,'linewidth',mw)];
        s=sprintf('%s : %d (%.1f%%)',labels{n},length(i),length(i)/size(p,1)*100);
        label={label{:},s};
    end
else
    i=find(q==qval);
    if ~isempty(i)
        plotpath(t(i),p(i,:),style,color,font,opt);
        h=plot(p(i,1),p(i,2),mark,'color',cs{qval},'markersize',msiz,'linewidth',mw);
        s=sprintf('%s : %d (%.1f%%)',labels{qval},length(i),length(i)/size(p,1)*100);
        label={s};
    end
end
if flag&~isempty(h), set(legend(h,label,4),'fontname',font); end

% plot path --------------------------------------------------------------------
function plotpath(t,p,style,color,font,opt)
tint=10;
plot(p(:,1),p(:,2),style,'color',color);
if ~flag|opt<1, return; end
for i=find(mod(t(1:end-1),tint)==floor(tint/2))' % add arrow
    pp=mean(p(i:i+1,1:2)); dp=p(i+1,1:2)-p(i,1:2); if t(i+1)<t(i), dp=-dp; end
    if norm(dp)<1, continue; end
    text(pp(1),pp(2),'\leftarrow','horizontal','center','vertical','middle',...
         'rotation',atan2(-dp(2),-dp(1))*180/pi,'color',color,'fontname',font,...
         'clipping','on');
end
if opt<2, return; end
for i=find(mod(t(1:end-1),tint)==0)' % add time label
    hh=floor(t(i)/3600); m=floor((t(i)-hh*3600)/60);
    s=sprintf('  %02d:%02d:%02d  ',hh,m,mod(t(i),60));
    if p(i,2)<p(i+1,2), hp='right'; else hp='left'; end
    text(p(i,1),p(i,2),s,'horizontal',hp,'fontname',font,'clipping','on');
end

% lat/lon/h to ecef ------------------------------------------------------------
function [r,E]=epos(pos)
RE=6378137; F=1/298.257223563;
pos(1:2)=pos(1:2)*pi/180;
sinp=sin(pos(1)); cosp=cos(pos(1)); sinl=sin(pos(2)); cosl=cos(pos(2));
e2=F*(2-F); v=RE/sqrt(1-e2*sinp^2);
r=[(v+pos(3))*cosp*[cosl;sinl];(v*(1-e2)+pos(3))*sinp];
E=[-sinl,cosl,0;-sinp*cosl,-sinp*sinl,cosp;cosp*cosl,cosp*sinl,sinp];

% set range --------------------------------------------------------------------
function setrange
pos=get(gcf,'position');
p=get(gca,'userdata'); if ~isstruct(p), return; end

% moving axis ------------------------------------------------------------------
function moveaxis
[p.pos,axp]=axpos; p.pnt=[0,0]; p.mov=0;
p.xl=xlim; p.yl=ylim; xs=p.xl(2)-p.xl(1); ys=p.yl(2)-p.yl(1);
if xs/ys<p.pos(3)/p.pos(4), p.xl=p.xl+[-1,1]*(ys*p.pos(3)/p.pos(4)-xs)/2;
else p.yl=p.yl+[-1,1]*(xs*p.pos(4)/p.pos(3)-ys)/2; end
set(gca,'position',axp,'userdata',p); xlim(p.xl); ylim(p.yl);

function [p,axp]=axpos
m=[40,20,20,25]; % margin [left,bottom,right,top]
pos=get(gcf,'position'); if any(pos(3:4)<=0), p=[m(1:2),100,100]; return; end
p=[m(1:2),max(pos(3:4)-m(1:2)-m(3:4),[100,100])];
axp=[p(1:2)./pos(3:4),p(3:4)./pos(3:4)];

function cb_resize
legend('ResizeLegend');
p=get(gca,'userdata'); if ~isstruct(p)|~isfield(p,'mov'), return; end
[pos,axp]=axpos; xs=p.xl(2)-p.xl(1); ys=p.yl(2)-p.yl(1);
p.xl=p.xl+[-1,1]*xs*(pos(3)/p.pos(3)-1)/2;
p.yl=p.yl+[-1,1]*ys*(pos(4)/p.pos(4)-1)/2;
p.pos=pos;
set(gca,'position',axp,'userdata',p); xlim(p.xl); ylim(p.yl);

function cb_btndown
p=get(gca,'userdata'); if ~isstruct(p)|~isfield(p,'mov'), return; end
alt=strcmp(get(gcf,'selectiontype'),'alt');
if alt, p.mov=2; setcurzoom; else p.mov=1; set(gcf,'pointer','fleur'); end
p.pnt=get(gcf,'currentpoint'); p.xl=xlim; p.yl=ylim;
set(gca,'userdata',p);

function cb_btndrag
p=get(gca,'userdata'); if ~isstruct(p)|~isfield(p,'mov')|p.mov<=0, return; end
siz=get(gcf,'position'); pos=get(gca,'position');
dp=(get(gcf,'currentpoint')-p.pnt)./siz(3:4)./pos(3:4);
switch p.mov
case 1, p.xl=p.xl-(p.xl(2)-p.xl(1))*dp(1); % move axis
        p.yl=p.yl-(p.yl(2)-p.yl(1))*dp(2);
case 2, p.xl=p.xl(1)+(p.xl(2)-p.xl(1))*(0.5+[-0.5,0.5]*0.1^dp(2)); % zoom axis
        p.yl=p.yl(1)+(p.yl(2)-p.yl(1))*(0.5+[-0.5,0.5]*0.1^dp(2));
end
xlim(p.xl); ylim(p.yl);

function cb_btnup
p=get(gca,'userdata'); if ~isstruct(p)|~isfield(p,'mov'), return; end
p.mov=0; p.xl=xlim; p.yl=ylim; set(gca,'userdata',p);
set(gcf,'pointer','arrow');

function setcurzoom
o=nan;
p=[o,o,o,o,o,o,o,2,1,2,o,o,o,o,o,o
   o,o,o,o,o,o,2,1,1,1,2,o,o,o,o,o
   o,o,o,o,o,2,1,1,1,1,1,2,o,o,o,o
   o,o,o,o,2,1,1,1,1,1,1,1,2,o,o,o
   o,o,o,2,1,1,1,1,1,1,1,1,1,2,o,o
   o,o,o,2,2,2,2,2,1,2,2,2,2,2,o,o
   o,o,o,o,o,o,o,2,1,2,o,o,o,o,o,o
   o,o,o,o,o,o,o,2,1,2,o,o,o,o,o,o
   o,o,o,o,o,2,2,2,1,2,2,2,o,o,o,o
   o,o,o,o,o,2,1,1,1,1,1,2,o,o,o,o
   o,o,o,o,o,2,2,2,1,2,2,2,o,o,o,o
   o,o,o,o,o,o,o,2,1,2,o,o,o,o,o,o
   o,o,o,o,o,2,2,2,1,2,2,2,o,o,o,o
   o,o,o,o,o,2,1,1,1,1,1,2,o,o,o,o
   o,o,o,o,o,o,2,1,1,1,2,o,o,o,o,o
   o,o,o,o,o,o,o,2,1,2,o,o,o,o,o,o];
set(gcf,'pointer','custom','pointershapecdata',p,'pointershapehotspot',[9,9])
